﻿// MZpack API demo
// Copyright © 2022 MZpack Co.
// www.mzpack.pro
// WARNING: to compile this sample in NinjaTrader 8 
// 1. Remove '#if APISAMPLE' and '#endif' directives
// 2. Use your own name space, for example NinjaTrader.NinjaScript.Strategies.MyNamespace
#if APISAMPLE
using System;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Windows;
using System.Windows.Controls;
using MZpack.NT8.Algo;
using MZpack.NT8.Algo.Extensions.Entries;
using NinjaTrader.Data;
using NinjaTrader.Gui;

namespace NinjaTrader.NinjaScript.Strategies.MZpackAPISamples
{
    /// <summary>
    /// Demonstrates MZpack Control Panel with custom and built-in controls.
    /// There are two ways to add controls to MZpack Control Panel:
    /// 1. Adding [ControlPanel] attribute to a public property. These types can be added: bool, enum.
    /// 2. Using CreateControlPanelElements() overrider to create any controls with custom handlers.
    /// 
    /// The strategy: 
    /// Opens LONG for Up-bar, opens SHORT for Down-bar.
    /// The strategy is OnBarClose.
    /// </summary>
    [CategoryOrder("Common", 1)]
    [CategoryOrder("Position", 2)]
    public class ControlPanel : MZpackStrategyBase
    {
        // Entry name
        static readonly string ENTRY_NAME = @"CP";
        //
        ControlPanelAlgoStrategy algoStrategy;
        // Custom elements for MZpack Control Panel
        Button showPatternsButton;
        ComboBox directionComboBox;
        
        // Custom algo strategy that is controlled from chart UI
        class ControlPanelAlgoStrategy : MZpack.NT8.Algo.Strategy
        {
            public ControlPanelAlgoStrategy(string name, ControlPanel MZpackStrategy) : base(name, MZpackStrategy)
            {
                OppositePatternAction = MZpackStrategy.OppositePatternAction;
                LogLevel = MZpackStrategy.LogLevel; // Set log level of algo from strategy UI
                LogTarget = MZpackStrategy.LogTarget; // Set log target of algo from strategy UI
                LogTime = MZpackStrategy.LogTime;
            }
        }

        public ControlPanel() : base()
        {
            // Create MZpack algo strategy object to use built-in pattern and position management
            OnCreateAlgoStrategy = new OnCreateAlgoStrategyDelegate(CreateAlgoStrategy);
            // Adjust the width of MZpack Control Panel
            ControlPanelWidth = 250;
        }

        protected MZpack.NT8.Algo.Strategy CreateAlgoStrategy()
        {
            algoStrategy = new ControlPanelAlgoStrategy(@"Control Panel", this);
            return algoStrategy;
        }

        // Create custom controls  
        public override UIElement[] CreateControlPanelElements()
        {
            UIElement[] ui = new UIElement[3];

            // ComboBox with label
            ui[0] = new Label() 
            { 
                Content = "Direction", 
                Foreground = ChartControl.Properties.ChartText, 
                Margin = new Thickness(ControlPanelAttribute.UIELEMENT_LEFT_MARGIN, 0, ControlPanelAttribute.UIELEMENT_RIGHT_MARGIN, ControlPanelAttribute.UIELEMENT_BOTTOM_MARGIN) 
            };
            ui[1] = directionComboBox = new ComboBox()
            {
                Name = "DirectionComboBox",
                Margin = new Thickness(ControlPanelAttribute.UIELEMENT_LEFT_MARGIN, 0, ControlPanelAttribute.UIELEMENT_RIGHT_MARGIN, ControlPanelAttribute.UIELEMENT_BOTTOM_MARGIN),
                ItemsSource = Enum.GetValues(typeof(SignalDirection)),
                SelectedValue = Position_Direction
            };

            // Button
            ui[2] = showPatternsButton = new Button()
            {
                Name = "ShowPatternsButton",
                Content = getShowPatternsText()
            };

            return ui;
        }

        // Attach event handlers to controls
        public override void ControlPanel_AttachEventHandlers()
        {
            if (directionComboBox != null) directionComboBox.SelectionChanged += DirectionComboBox_SelectionChanged;
            if (showPatternsButton != null) showPatternsButton.Click += ShowPatternsButton_Click;
        }

        // Detach event handlers from controls
        public override void ControlPanel_DetachEventHandlers() 
        {
            if (directionComboBox != null) directionComboBox.SelectionChanged -= DirectionComboBox_SelectionChanged;
            if (showPatternsButton != null) showPatternsButton.Click -= ShowPatternsButton_Click;
        }

        public void ShowPatternsButton_Click(object sender, RoutedEventArgs e)
        {
            ShowPatterns = !ShowPatterns;
            (sender as Button).Content = getShowPatternsText();  // Update button text
        }

        public void DirectionComboBox_SelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e)
        {
            Position_Direction = (SignalDirection)(sender as ComboBox).SelectedItem;
        }

        string getShowPatternsText()
        {
            if (ShowPatterns) return @"Hide patterns"; else return @"Show patterns";
        }

        protected override void OnStateChange()
        {
            // Base OnStateChange() call is required
            base.OnStateChange();

            lock (Sync) // Sync state handler
            {
                if (State == State.SetDefaults)
                {
                    BarsRequiredToTrade = 1;

                    // Default values 
                    Operating = StrategyOperating.Auto;
                    OppositePatternAction = OppositePatternAction.Reverse;
                    EntryMarkup = EntryMarkup.MarkerAndText;
                    ControlPanelShow = true;

                    // Position allowed directions, quantity and stop/target
                    Position_Direction = SignalDirection.Any;
                    Position_Quantity = 2;
                    Position_StopLoss = 6;
                    Position_ProfitTarget = 12;
                }
                else
                if (State == State.Configure)
                {
                    EntriesPerDirection = 1;

                    // Create entry
                    Entry[] entries = new Entry[EntriesPerDirection];
                    entries[0] = new Entry(Strategy)
                    {
                        EntryMethod = EntryMethod.Market,
                        Quantity = Position_Quantity,
                        SignalName = ENTRY_NAME,
                        StopLossTicks = Position_StopLoss,
                        ProfitTargetTicks = Position_ProfitTarget,
                    };

                    // Create pattern
                    Pattern pattern = new Pattern(Strategy,
                        Logic.And, // Logic of root node of signals tree
                        null, // No range for signals
                        true);  // Short-cuircit for AND evaluation

                    // Create signals logical tree
                    pattern.Signals.Root.AddChild(new UpDownBarSignal(Strategy));

                    // Initialize strategy
                    Strategy.Initialize(pattern, entries, 3);
                }
            }
        }

        // UpDown bar signal. Set LONG direction for Up-bar, set SHORT direction for Down-bar.
        class UpDownBarSignal : Signal
        {
            public UpDownBarSignal(MZpack.NT8.Algo.Strategy strategy) : base(strategy, MarketDataSource.Level1, SignalCalculate.OnBarClose, true)
            {
            }

            public override void OnCalculate(MarketDataEventArgs e, int barIdx, SignalDirection allowed)
            {
                SignalDirection direction = SignalDirection.None;

                // Down-bar    
                if (Signal.IsLongAllowed(allowed) && Strategy.MZpackStrategy.Open[1] > Strategy.MZpackStrategy.Close[1]) 
                    direction = Signal.ResolveDirection(SignalDirection.Short, allowed);
                else
                // Up-bar
                if (Signal.IsLongAllowed(allowed) && Strategy.MZpackStrategy.Open[1] < Strategy.MZpackStrategy.Close[1])
                    direction = Signal.ResolveDirection(SignalDirection.Long, allowed);

                if (Signal.IsDetermined(direction))
                {
                    Direction = direction;
                    ChartRange = new ChartRange() { MinBarIdx = barIdx, MaxBarIdx = barIdx };
                    Time = e.Time;
                    EntryPrice = e.Price;
                }
            }
        }

        // ----------------------------- MZpack
        [Display(Name = "Operating", GroupName = "MZpack", Order = 2, Description = "")]
        [ControlPanel]  // Override inherited property and use ControlPanel attribute to add it to MZpack ControlPanel
        public override StrategyOperating Operating { get; set; }

        [Display(Name = "Entry/Exit: markup", GroupName = "MZpack", Order = 30, Description = "")]
        [ControlPanel]  // Override inherited property and use ControlPanel attribute to add it to MZpack ControlPanel
        public override EntryMarkup EntryMarkup { get; set; }

        [Display(Name = "Pattern dashboard: show", GroupName = "MZpack", Order = 40, Description = "")]
        [ControlPanel]  // Override inherited property and use ControlPanel attribute to add it to MZpack ControlPanel
        public override bool ShowPatternsDashboard { get; set; }

        [Display(Name = "Pattern dashboard: legend", GroupName = "MZpack", Order = 50, Description = "")]
        [ControlPanel]  // Override inherited property and use ControlPanel attribute to add it to MZpack ControlPanel
        public override bool DashboardGridShowLegend { get; set; }

        [Display(Name = "Pattern dashboard: position", GroupName = "MZpack", Order = 51, Description = "")]
        [ControlPanel]  // Override inherited property and use ControlPanel attribute to add it to MZpack ControlPanel
        public override DashboardViewPosition DashboardGridViewPosition { get; set; }

        // ----------------------------- Common
        [Display(Name = "Opposite Pattern Action", GroupName = "Common", Order = 1, Description = "")]
        [ControlPanel]  // Add property to MZpack ControlPanel using ControlPanel attribute
        public OppositePatternAction OppositePatternAction
        { get; set; }

        // ----------------------------- Position
        SignalDirection position_Direction = SignalDirection.Any;  // declare private member for holding property value
        [Display(Name = "Direction", GroupName = "Position", Order = 0, Description = "")]
        [ControlPanel()]  // Add property to MZpack ControlPanel using ControlPanel attribute
        public SignalDirection Position_Direction { get { return position_Direction; } set { position_Direction = value; if (Strategy != null && Strategy.Pattern != null) Strategy.Pattern.AllowedDirection = value; } }

        [Display(Name = "Quantity", GroupName = "Position", Order = 1, Description = "")]
        [Range(1, int.MaxValue)]
        public int Position_Quantity
        { get; set; }

        [Display(Name = "Stop loss, ticks", GroupName = "Position", Order = 2, Description = "")]
        [Range(1, int.MaxValue)]
        public int Position_StopLoss
        { get; set; }

        [Display(Name = "Profit target, ticks", GroupName = "Position", Order = 3, Description = "")]
        [Range(1, int.MaxValue)]
        public int Position_ProfitTarget
        { get; set; }
    }
}
#endif
