﻿// MZpack 3 Pro API demo
// Copyright © 2021 MZpack Co.
// www.mzpack.pro
// WARNING: to compile this sample in NinjaTrader 8 remove '#if APISAMPLE' and '#endif' directives
#if APISAMPLE
using System;
using System.Collections.Generic;
using MZpack;
using MZpack.NT8;
using MZpack.NT8.Algo;
using MZpack.NT8.Algo.Indicators;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using NinjaTrader.Gui;
using NinjaTrader.Data;

namespace NinjaTrader.NinjaScript.Strategies.MZpackAPISamples
{
    /// <summary>
    /// Demonstrates how to access data of MZpack mzFootprint indicator (StrategyFootprintIndicator class).
    /// See installation folder for the source code.
    /// </summary>
    [CategoryOrder("Settings", 1)]
    public class DataAccess_mzFootprint : MZpackStrategyBase
    {
        StrategyFootprintIndicator footprintIndicator;


        public DataAccess_mzFootprint() : base()
        {
            // Set OnCreateIndicators delegate
            OnCreateIndicators = new OnCreateIndicatorsDelegate(CreateIndicators);

            // Set OnBarCloseHandler for our on bar close strategy
            OnBarCloseHandler = new OnTickDelegate(StrategyOnBarCloseHandler);
        }

        /// <summary>
        /// Create indicator here and return the list with one item as a result.
        /// </summary>
        /// <returns>Indicators list</returns>
        protected List<TickIndicator> CreateIndicators()
        {
            // Initialize new indicators list
            List<TickIndicator> indicators = new List<TickIndicator>();

            // Create StrategyFootprintIndicator instance 
            footprintIndicator = new StrategyFootprintIndicator(this, @"Footprint")
            {
                // Override defaults if required
                // See IFootprintIndicator interface
                LeftFootprintStyle = FootprintStyle.Delta,
                RightFootprintStyle = FootprintStyle.Volume,
                
                //
                TicksPerLevel = 1,

                // Show developing levels (POC, VAH, VAL) of session volume profile
                ShowSessionPOC = true,
                ShowSessionVA = true,
                SessionDailyProfileMode = SessionDailyProfileMode.Session,
                SessionVAIsDeveloping = true,
                
                // Bar volume profile levels
                ShowBarVA = true,
                BarVAPercentage = 70,
                ShowBarPOC = true,
                ShowBarPOCCount = 3,  // Show 3 POCs of the bar
                
                // Bar stat
                ShowBarVolume = true,
                ShowBarDelta = true,
                ShowBarAbsoluteDeltaAverage = false,
                ShowBarMinMaxDelta = false,
                ShowBarDeltaPercent = false,
                ShowBarCOT = true,
                ShowBarRatioNumbers = true,
                BarRatioNumbersBoundsLow = 0.4,
                BarRatioNumbersBoundsHigh = 77,

                // Imbalances (diagonal)
                ShowImbalance = true,
                ImbalancePercentage = 200,  // 200%
                ImbalanceFilter = 20, // Imbalance filter in contracts
                ImbalanceMarker = FootprintImbalanceMarker.Always, // Always show imbalance markers
                ImbalanceMarkerPosition = FootprintImbalanceMarkerPosition.Outer,
                ImbalanceHighlightValues = false,  // Don't highlight imbalance values since ImbalanceMarker = FootprintImbalanceMarker.Always

                // Imbalance SR Zones
                ShowImbalanceSRZones = true,
                ImbalanceSRZonesConsecutiveLevels = 2,
                ImbalanceSRZonesVolumeFilter = 200,  
                ImbalanceSRZoneEnding = SRZoneEnding.ByBarPOC,
                ImbalanceSRZonesBreakOnSession = true,
                ImbalanceSRZoneAlert = true,

                // Absorptions (diagonal)
                ShowAbsorption = true,
                AbsorptionPercentage = 300,  // 300%
                AbsorptionDepth = 2, // Depth is 2 ticks
                AbsorptionFilter = 200,  // Absorption filter in contracts

                // Absorption SR Zones
                ShowAbsorptionSRZones = false,

                // Unfinished Auction
                ShowUnfinishedAuction = true
            };

            // Cluster zones
            footprintIndicator.FootprintPresentation[FootprintBaseMVC.RIGHT_FOOTPRINT].ClusterZonesShow = true;
            footprintIndicator.FootprintPresentation[FootprintBaseMVC.RIGHT_FOOTPRINT].ClusterZonesFilterMin = 4000;
            footprintIndicator.FootprintPresentation[FootprintBaseMVC.RIGHT_FOOTPRINT].ClusterZonesFilterMax = -1;
            footprintIndicator.FootprintPresentation[FootprintBaseMVC.RIGHT_FOOTPRINT].ClusterZonesBreakOnSession = false;
            footprintIndicator.FootprintPresentation[FootprintBaseMVC.RIGHT_FOOTPRINT].ClusterZonesStyle = LevelPresentationStyle.Line;

            // Statistic Grid
            footprintIndicator.StatisticGridShow = true;
            footprintIndicator.StatisticGridShowArr[FootprintBaseMVC.STATGRID_VOLUME] = true;
            footprintIndicator.StatisticGridShowArr[FootprintBaseMVC.STATGRID_BUYVOLUME] = false;
            footprintIndicator.StatisticGridShowArr[FootprintBaseMVC.STATGRID_SELLVOLUME] = false;
            footprintIndicator.StatisticGridShowArr[FootprintBaseMVC.STATGRID_TRADES] = true;
            footprintIndicator.StatisticGridShowArr[FootprintBaseMVC.STATGRID_DELTACUMULATIVE] = true;
            footprintIndicator.StatisticGridShowArr[FootprintBaseMVC.STATGRID_DELTARATE] = true;

            // Statistic Grid: Predicted values
            footprintIndicator.StatisticGridPredictedValuesShow = true;
            footprintIndicator.StatisticGridPredictedValuesGaugeShow = true;

            // Add indicator to the list
            indicators.Add(footprintIndicator);  

            return indicators;
        }

        protected override void OnStateChange()
        {
            // Base OnStateChange() call is required
            base.OnStateChange();

            lock (Sync) // Sync OnStateChange handler
            {
                if (State == State.SetDefaults)
                {
                    EntriesPerDirection = 1;
                    BarsRequiredToTrade = 1;

                    // To print data from hisotrical bars
                    EnableBacktesting = true;
                }
                else if (State == State.Configure)
                {
                    footprintIndicator.Calculate = Calculate.OnBarClose;  // Calculate.OnBarClose is required for mzFootprint if the logic is in OnBarCloseHandler
                }
            }
        }

        /// <summary>
        /// The handler triggered on bar close. currentBarIdx is the index of just closed bar.
        /// Parameter e is the first tick of just opened bar.
        /// </summary>
        protected void StrategyOnBarCloseHandler(MarketDataEventArgs e, int currentBarIdx)
        {
            if (footprintIndicator.FootprintBars.ContainsKey(currentBarIdx))
            {
                IFootprintBar bar = footprintIndicator.FootprintBars[currentBarIdx];

                Print("=======================================");
                Print("Bar: " + bar.BarIdx);
                //Print("Is first bar of session: " + bar.IsFirstBarOfSession);

                Print("Range, ticks: " + bar.RangeTicks);
                Print("Range, levels: " + bar.RangeLevels);  // Range of bar in levels according to TicksPerLevel value.
                Print("Volume: " + footprintIndicator.FromInternalVolume(bar.Volume));  // For Cryptocurrencies use FromInternalVolume() to convert all internal volumes to Crypto volumes or to apply Non-Crypto volumes divider. 
                Print("Delta: " + bar.Delta);  // Total Delta in the bar
                Print("COT High: " + bar.COTHigh);
                Print("COT Low: " + bar.COTLow);

                // Some predicted values (available in real-time state only)
                if (State == State.Realtime)
                {
                    Print("Volume predicted: " + bar.VolumePredicted);  // Total Delta in the bar
                    Print("COT High predicted: " + bar.COTHighPredicted);
                    Print("COT Low predicted: " + bar.COTLowPredicted);
                }

                // Single (1st) POC, VAH, VAL
                Print(string.Format("POC {0} @{1}; VAH @{2}; VAL @{3}", bar.POCVolume, bar.POC, bar.VAH, bar.VAL));

                // Bar POCs ordered by volume 
                int i = 1;
                foreach (var poc in bar.MultiplePOC)
                {
                    Print(string.Format("POC #{0} @{1} volume: {2}", i, poc.Key, poc.Value));
                    if (i == footprintIndicator.ShowBarPOCCount)
                        break;
                    i++;
                }

                // Delta Rate
                Print("Delta Rate: " + bar.DeltaRate);
                Print(string.Format("Delta Rate High: {0}; Low: {1}", bar.DeltaRateHigh, bar.DeltaRateLow));

                // Session volume profile
                ISession currentSession = footprintIndicator.Sessions.LastOrDefault();
                if (currentSession != null)
                {
                    Print("Session POC: @" + currentSession.POC);
                    Print("Session VAH: @" + currentSession.VAH);
                    Print("Session VAL: @" + currentSession.VAL);
                }

                // Sell Imbalances
                foreach (var item in bar.Imbalances[(int)TradeSide.Bid])
                {
                    long volume = item.Value;
                    double price = item.Key;
                    Print(string.Format("Sell Imbalance : {0} @{1}", volume, price));
                }
                // Buy Imbalances
                foreach (var item in bar.Imbalances[(int)TradeSide.Ask])
                {
                    long volume = item.Value;
                    double price = item.Key;
                    Print(string.Format("Buy Imbalance: {0} @{1}", volume, price));
                }

                // Buy Imbalance Zones of the bar
                foreach (ISRZone zone in bar.ImbalanceSRZones.Zones[(int)TradeSide.Ask])
                {
                    long volume = zone.Volume;
                    double high = zone.Hi;
                    double low = zone.Lo;
                    Print(string.Format("Buy Imbalance Zone: {0} @{1}-{2}", volume, high, low));
                }
                // Sell Imbalance Zones of the bar
                foreach (ISRZone zone in bar.ImbalanceSRZones.Zones[(int)TradeSide.Bid])
                {
                    long volume = zone.Volume;
                    double high = zone.Hi;
                    double low = zone.Lo;
                    Print(string.Format("Sell Imbalance Zone: {0} @{1}-{2}", volume, high, low));
                }

                // Buy Imbalance Zones total volume using Linq
                if (bar.ImbalanceSRZones.HasZones(TradeSide.Ask))
                {
                    long total = bar.ImbalanceSRZones.Zones[(int)TradeSide.Ask].Sum(z => z.Volume);
                    Print("Buy Imbalance Zones total volume: " + total);
                }

                // Live (actual) and Ended by the bar Buy and Sell Imbalance Zones. 
                TradeSide side = TradeSide.Ask;  // Start from Ask side
                while (true)
                {
                    string sideStr = side == TradeSide.Bid ? "Sell" : "Buy";

                    // Live zones
                    List<ISRZone> liveZones = footprintIndicator.LiveSRZones(SRZoneType.Imbalance, side);
                    foreach (var zone in liveZones)
                    {
                        long volume = zone.Volume;
                        double high = zone.Hi;
                        double low = zone.Lo;
                        Print(string.Format("Live {0} Imbalance Zone: {1} @{2}-{3}", sideStr, volume, high, low));
                    }
                    // Ended
                    foreach (ISRZone zone in bar.ImbalanceSRZones.EndedZones[(int)side])
                    {
                        long volume = zone.Volume;
                        double high = zone.Hi;
                        double low = zone.Lo;
                        Print(string.Format("Ended {0} Imbalance Zone: {1} @{2}-{3}", sideStr, volume, high, low));
                    }

                    if (side == TradeSide.Ask)
                        side = TradeSide.Bid; // The next side is Bid
                    else
                        break;
                }

                // Live Cluster zones
                foreach (IClusterZone zone in footprintIndicator.LiveClusterZones(FootprintBaseMVC.RIGHT_FOOTPRINT))
                    Print(string.Format("Live Cluster Zone: value: {0} @{1} bars: {2}-...", zone.Value, zone.Price, zone.BeginBar.BarIdx));

                // Ended Cluster zones
                foreach (IClusterZone zone in footprintIndicator.EndedClusterZones(FootprintBaseMVC.RIGHT_FOOTPRINT))
                    Print(string.Format("Ended Cluster Zone: value: {0} @{1} bars: {2}-{3}", zone.Value, zone.Price, zone.BeginBar.BarIdx, zone.EndBar.BarIdx));
            }
        }

    }
}
#endif