﻿// MZpack 3 Pro API demo
// Copyright © 2021 MZpack Co.
// www.mzpack.pro
// WARNING: to compile this sample in NinjaTrader 8 remove '#if APISAMPLE' and '#endif' directives
#if APISAMPLE
using System;
using System.Collections.Generic;
using MZpack;
using MZpack.NT8;
using MZpack.NT8.Algo;
using MZpack.NT8.Algo.Indicators;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using NinjaTrader.Gui;
using NinjaTrader.Data;

namespace NinjaTrader.NinjaScript.Strategies.MZpackAPISamples
{
    /// <summary>
    /// Demonstrates how to access data of MZpack mzVolumeDelta indicator (StrategyVolumeDeltaIndicator class).
    /// See installation folder for the source code.
    /// </summary>
    [CategoryOrder("Settings", 1)]
    public class DataAccess_mzVolumeDelta : MZpackStrategyBase
    {
        StrategyVolumeDeltaIndicator volumeDeltaIndicator;


        public DataAccess_mzVolumeDelta() : base()
        {
            // Set OnCreateIndicators delegate
            OnCreateIndicators = new OnCreateIndicatorsDelegate(CreateIndicators);

            // Set OnBarCloseHandler for our on bar close strategy
            OnBarCloseHandler = new OnTickDelegate(StrategyOnBarCloseHandler);
        }

        /// <summary>
        /// Create indicator here and return the list with one item as a result.
        /// </summary>
        /// <returns>Indicators list</returns>
        protected List<TickIndicator> CreateIndicators()
        {
            // Initialize new indicators list
            List<TickIndicator> indicators = new List<TickIndicator>();

            // Create StrategyVolumeDeltaIndicator instance 
            volumeDeltaIndicator = new StrategyVolumeDeltaIndicator(this, @"Volume Delta")
            {
                // Override defaults if required
                // See IVolumeDeltaIndicator interface
                VolumeDeltaMode = VolumeDeltaMode.Delta, // Delta mode
                DeltaMode = DeltaMode.Cumulative,  // Cumulative Delta
                TradeFilterMin = 0,  // Filters
                TradeFilterMax = 10,
                ZeroLine = true  // Plot zero Delta line
            };
            // Add indicator to the list
            indicators.Add(volumeDeltaIndicator);  

            return indicators;
        }

        protected override void OnStateChange()
        {
            // Base OnStateChange() call is required
            base.OnStateChange();

            lock (Sync) // Sync OnStateChange handler
            {
                if (State == State.SetDefaults)
                {
                    EntriesPerDirection = 1;
                    BarsRequiredToTrade = 1;

                    // Set defaults
                    TradeFilterMin = 0;
                    TradeFilterMax = -1;

                    // This is required for backtesting in Strategy Analyzer, set false if you don't need it to reduce loading time
                    EnableBacktesting = true;
                }
                else if (State == State.Configure)
                {
                    // When State = State.Configure we can set indicator attributes from UI properties
                    volumeDeltaIndicator.TradeFilterMin = TradeFilterMin;
                    volumeDeltaIndicator.TradeFilterMax = TradeFilterMax;
                }
            }
        }

        /// <summary>
        /// The handler triggered on bar close. currentBarIdx is the index of just closed bar.
        /// Parameter e is the first tick of just opened bar.
        /// </summary>
        protected void StrategyOnBarCloseHandler(MarketDataEventArgs e, int currentBarIdx)
        {
            if (volumeDeltaIndicator.VolumeDeltaBars.ContainsKey(currentBarIdx))
            {
                IVolumeDeltaBar bar = volumeDeltaIndicator.VolumeDeltaBars[currentBarIdx];

                Print("=======================================");
                Print("Bar: " + currentBarIdx);
                Print("Volume: " + volumeDeltaIndicator.FromInternalVolume(bar.Volume));  // For Cryptocurrencies use FromInternalVolume() to convert all internal volumes to Crypto volumes or apply Non-Crypto volumes divider. 
                Print("Sell volume: " + bar.VolumeBySide(TradeSide.Bid));
                Print("Buy volume: " + bar.VolumeBySide(TradeSide.Ask));
                Print("Iceberg volume: " + bar.IcebergVolume);
                Print("Iceberg volume on bid: " + bar.IcebergVolumeBySide(TradeSide.Bid));
                Print("Iceberg volume on offer: " + bar.IcebergVolumeBySide(TradeSide.Ask));
                Print("Delta: " + bar.Delta);  // Total Delta in the bar
                Print("Open Delta: " + bar.OpenDelta);  // Bar open delta or cumulative Delta of previous bar
                Print("Cumulative Delta: " + bar.CumulativeDelta);  // Cumulative delta of the bar
                Print("Max Delta: " + bar.DeltaHi);  // Maximal Delta in the bar
                Print("Min Delta: " + bar.DeltaLo);  // Minimal Delta in the bar
                Print("Max cumulative Delta: " + (bar.DeltaHi + bar.OpenDelta).ToString());  // Maximal cumulative Delta in the bar
                Print("Min cumulative Delta: " + (bar.DeltaLo + bar.OpenDelta).ToString());  // Minimal cumulative Delta in the bar
            }
        }


        [Display(Name = "Trade: min", GroupName = "Settings", Order = 0, Description = "")]
        [Range(0, double.MaxValue)]
        [NinjaScriptProperty]
        public double TradeFilterMin { get; set; }

        [Display(Name = "Trade: max", GroupName = "Settings", Order = 1, Description = "")]
        [Range(-1, double.MaxValue)]
        [NinjaScriptProperty]
        public double TradeFilterMax { get; set; }
    }
}
#endif