﻿// MZpack 3 Pro API demo
// Copyright © 2020 MZpack Co.
// www.mzpack.pro
// WARNING: to compile this sample in NinjaTrader 8 remove '#if APISAMPLE' and '#endif' directives
#if APISAMPLE
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using MZpack;
using MZpack.NT8;
using MZpack.NT8.Algo;
using MZpack.NT8.Algo.Extensions.Entries;
using MZpack.NT8.Algo.Extensions.Trails;
using MZpack.NT8.Algo.Indicators;
using MZpack.NT8.Algo.Signals;

namespace NinjaTrader.NinjaScript.Strategies.MZpackAPISamples
{
    /// <summary>
    /// Demonstrates algo strategy class MZpack.NT8.Algo.Strategy with proprietary ATM and built-in TradesClusterSignal class.
    /// </summary>
    public class MZpackAlgoStrategy0 : MZpackStrategyBase
    {
        // Indicators
        StrategyBigTradeIndicator bigTradeIndicator;

        // Signal name
        static readonly string SIGNAL_NAME = @"Cluster Entry";


        public MZpackAlgoStrategy0() : base()
        {
            // Create MZpack algo strategy object to use built-in pattern and position management
            OnCreateAlgoStrategy = new OnCreateAlgoStrategyDelegate(CreateAlgoStrategy);

            // Set OnCreateIndicators delegate
            OnCreateIndicators = new OnCreateIndicatorsDelegate(CreateIndicators);
        }

        // Create MZpack Algo Strategy object to support pattern and trade management (ATM).
        protected MZpack.NT8.Algo.Strategy CreateAlgoStrategy()
        {
            return new MZpack.NT8.Algo.Strategy(@"MZpack Algo Strategy #0", this) 
            { 
                OppositePatternAction = OppositePatternAction.Reverse, // Reverse position on validated opposite pattern
                LogLevel = LogLevel, // Set log level of algo from strategy UI
                LogTarget = LogTarget, // Set log target of algo from strategy UI
                LogTime = LogTime
            };
        }

        // Create indicators here and return them in the list as a result.
        protected List<TickIndicator> CreateIndicators()
        {
            // Initialize new indicators list
            List<TickIndicator> indicators = new List<TickIndicator>();

            // Create BigTrade indicator
            bigTradeIndicator = new StrategyBigTradeIndicator(this, @"Clusters")
            {
                // Display filters
                FilterLogic = TradeFilterLogic.ALL,
                TradeFilterEnable = true,
                DomPressureFilterEnable = true,
                AggressionFilterEnable = false,
                // Presentation
                MarkerType = TradeMarkerType.Box,
                MaxShapeExtent = 10,
                ColorMode = ColorMode.Saturation,
                ShowTradePOC = true,
                VolumesPosition = VolumesPosition.OutsideRight,
                // Alerts
                UseBigTradeAlert = false
            };
            indicators.Add(bigTradeIndicator);

            return indicators;
        }

        protected override void OnStateChange()
        {
            // Base OnStateChange() call is required
            base.OnStateChange();

            lock (Sync) // Sync state handler
            {
                if (State == State.SetDefaults)
                {
                    // Default values for parameters
                    Common_TraceOrders = false;
                    // Position allowed directions, quantity and stop/target
                    Position_Direction = SignalDirection.Any;
                    Position_Quantity = 2;
                    Position_StopLoss = 18;
                    Position_ProfitTarget = 35;
                    // Break even
                    Position_IsBE = false;
                    Position_BEAfterTicks = 14;
                    Position_BEShiftTicks = 4;
                    // Trailing
                    Position_IsTrail = true;
                    Position_TrailAfter = 20;
                    Position_TrailDistance = 10;
                    Position_TrailStep = 2;
                    // Signals
                    BigTrade_TradeMin = 20;
                    BigTrade_TradeMax = 500;
                    BigTrade_DomPressureMin = 5;
                    BigTrade_ClusterBarRange = 4;
                    BigTrade_ClusterPriceRangeTicks = 4;
                    BigTrade_MinTotalVolume = 50;
                    BigTrade_MinTotalCount = 3;
                }
                else
            if (State == State.Configure)
                {
                    TraceOrders = Common_TraceOrders;
                    EntriesPerDirection = 1;

                    // Assign indicator's attributes from UI properties
                    bigTradeIndicator.TradeFilterMin = BigTrade_TradeMin;
                    bigTradeIndicator.TradeFilterMax = BigTrade_TradeMax;
                    bigTradeIndicator.DomPressureFilterMin = BigTrade_DomPressureMin;
                    bigTradeIndicator.Visible = true;

                    // Create entries
                    Entry[] entries = new Entry[EntriesPerDirection];
                    // Create Trail object
                    Trail trail = Position_IsTrail ? new Trail(Position_TrailAfter, Position_TrailDistance, Position_TrailStep) : null;
                    entries[0] = new Entry(Strategy)
                    {
                        EntryMethod = EntryMethod.Market,
                        Quantity = Position_Quantity,
                        SignalName = SIGNAL_NAME,
                        StopLossTicks = Position_StopLoss,
                        ProfitTargetTicks = Position_ProfitTarget,
                        IsBreakEven = Position_IsBE,
                        BreakEvenAfterTicks = Position_BEAfterTicks,
                        BreakEvenShiftTicks = Position_BEShiftTicks,
                        Trail = trail
                    };

                    // Pattern
                    Pattern pattern = new Pattern(Strategy,
                        Logic.Or, // Logic of root node of signals tree
                        null, // No range for signals
                        true);  // Short-cuircit for AND evaluation

                    // Create signals decision tree
                    // Signal for SHORT (cluster of buy trades)
                    pattern.Signals.Root.AddChild(
                        new TradesClusterSignal(Strategy, bigTradeIndicator, SignalDirection.Short)
                        {
                            Name = "Ask Clusters",
                            Side = TradeSide.Ask,
                            Range = new Range() { Bars = BigTrade_ClusterBarRange, Ticks = BigTrade_ClusterPriceRangeTicks, Logic = Logic.And },
                            MinVolume = BigTrade_MinTotalVolume,
                            MinCount = BigTrade_MinTotalCount,
                            VolumeCountLogic = Logic.And,
                            PartiallyVisibleMode = SignalPartiallyVisibleMode.OnPatternValidated
                        });
                    // Signal for LONG(cluster of sell trades)
                    pattern.Signals.Root.AddChild(
                        new TradesClusterSignal(Strategy, bigTradeIndicator, SignalDirection.Long)
                        {
                            Name = "Bid Clusters",
                            Side = TradeSide.Bid,
                            Range = new Range() { Bars = BigTrade_ClusterBarRange, Ticks = BigTrade_ClusterPriceRangeTicks, Logic = Logic.And },
                            MinVolume = BigTrade_MinTotalVolume,
                            MinCount = BigTrade_MinTotalCount,
                            VolumeCountLogic = Logic.And,
                            PartiallyVisibleMode = SignalPartiallyVisibleMode.OnPatternValidated
                        });

                    // Initialize strategy
                    Strategy.Initialize(pattern, entries, 3);
                }
            }
        }


        // --------------------------------- Common
        [Display(Name = "Trace orders", GroupName = "Common", Order = 21, Description = "")]
        public bool Common_TraceOrders
        { get; set; }

        // ----------------------------- Position
        [Display(Name = "Direction", GroupName = "Position", Order = 0, Description = "")]
        public SignalDirection Position_Direction
        { get; set; }

        [Display(Name = "Quantity", GroupName = "Position", Order = 1, Description = "")]
        [Range(1, int.MaxValue)]
        public int Position_Quantity
        { get; set; }

        [Display(Name = "Stop loss, ticks", GroupName = "Position", Order = 3, Description = "")]
        [Range(1, int.MaxValue)]
        public int Position_StopLoss
        { get; set; }

        [Display(Name = "Profit target, ticks", GroupName = "Position", Order = 5, Description = "")]
        [Range(1, int.MaxValue)]
        public int Position_ProfitTarget
        { get; set; }

        [Display(Name = "BE", GroupName = "Position", Order = 7, Description = "")]
        public bool Position_IsBE
        { get; set; }

        [Display(Name = "BE after, ticks", GroupName = "Position", Order = 10, Description = "")]
        [Range(1, int.MaxValue)]
        public int Position_BEAfterTicks
        { get; set; }

        [Display(Name = "BE shift, ticks", GroupName = "Position", Order = 11, Description = "")]
        public int Position_BEShiftTicks
        { get; set; }

        [Display(Name = "Trail entry", GroupName = "Position", Order = 12, Description = "")]
        public bool Position_IsTrail
        { get; set; }

        [Display(Name = "Trail after, ticks", GroupName = "Position", Order = 20, Description = "")]
        [Range(1, int.MaxValue)]
        public int Position_TrailAfter
        { get; set; }

        [Display(Name = "Trail distance, ticks", GroupName = "Position", Order = 21, Description = "")]
        [Range(1, int.MaxValue)]
        public int Position_TrailDistance
        { get; set; }

        [Display(Name = "Trail step, ticks", GroupName = "Position", Order = 22, Description = "")]
        [Range(1, int.MaxValue)]
        public int Position_TrailStep
        { get; set; }

        //--------------------------------------- Signals
        [Display(Name = "Trade: min volume", GroupName = "Signals", Order = 2, Description = "Trade min volume filter")]
        [Range(1, long.MaxValue)]
        public long BigTrade_TradeMin
        { get; set; } 

        [Display(Name = "Trade: max volume", GroupName = "Signals", Order = 3, Description = "Trade min volume filter")]
        [Range(1, long.MaxValue)]
        public long BigTrade_TradeMax
        { get; set; }

        [Display(Name = "Trade: DOM pressure min", GroupName = "Signals", Order = 5, Description = "Dom Pressure min volume filter")]
        [Range(0, long.MaxValue)]
        public long BigTrade_DomPressureMin
        { get; set; }

        [Display(Name = "Trade: cluster bar range", GroupName = "Signals", Order = 6, Description = "")]
        [Range(-1, int.MaxValue)]
        public int BigTrade_ClusterBarRange
        { get; set; }

        [Display(Name = "Trade: cluster price range, ticks", GroupName = "Signals", Order = 7, Description = "")]
        [Range(-1, int.MaxValue)]
        public int BigTrade_ClusterPriceRangeTicks
        { get; set; }

        [Display(Name = "Trade: min volume", GroupName = "Signals", Order = 9, Description = "")]
        [Range(1, long.MaxValue)]
        public long BigTrade_MinTotalVolume
        { get; set; }

        [Display(Name = "Trade: min count", GroupName = "Signals", Order = 9, Description = "")]
        [Range(1, int.MaxValue)]
        public int BigTrade_MinTotalCount
        { get; set; }

    }
}
#endif
