﻿// MZpack 3 Pro API demo
// Copyright © 2020 MZpack Co.
// www.mzpack.pro
// WARNING: to compile this sample in NinjaTrader 8 remove '#if APISAMPLE' and '#endif' directives
#if APISAMPLE
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using NinjaTrader.Data;
using MZpack;
using MZpack.NT8;
using NinjaTrader.Cbi;
using System.ComponentModel;
using System.Xml.Serialization;
using MZpack.NT8.Algo;
using MZpack.NT8.Algo.Indicators;
using NinjaTrader.Gui;
using MZpack.NT8.Algo.Extensions.Entries;
using MZpack.NT8.Algo.Extensions.Trails;

namespace NinjaTrader.NinjaScript.Strategies.MZpackAPISamples
{
    /// <summary>
    /// The strategy demonstrates using of patterns and built-in position management (proprietary ATM).
    /// </summary>
    /// <summary>
    /// WARNING: The strategy works on Market replay or live data only because it uses Level 2 data.
    /// </summary>
    /// <summary>
    /// ATTENTION: Indicator settings in UI are used only for displaying markers on the chart. Strategy uses its own internal settings for filters and logic.
    /// </summary>
    [CategoryOrder("Common", 10)]
    [CategoryOrder("Position", 20)]
    [CategoryOrder("Signals", 30)]
    [CategoryOrder("Filters", 40)]

    public class MZpackAlgoStrategy1 : MZpackStrategyBase
    {
        // Indicators
        [Browsable(false)] [XmlIgnore] public StrategyBigTradeIndicator BigTradeIndicator;
        [Browsable(false)] [XmlIgnore] public EMA EMAIndicator;
        [Browsable(false)] [XmlIgnore] public StrategyMarketDepthIndicator DOMImbalanceIndicator;

        // 
        int pendingLimitBarIdx = -1;
        
        // Unique names for signals
        static readonly string SIGNAL1 = @"Entry 1";
        static readonly string SIGNAL2 = @"Entry 2";


        public MZpackAlgoStrategy1() : base()
        {
            // Create MZpack algo strategy object to use built-in pattern and position management
            OnCreateAlgoStrategy = new OnCreateAlgoStrategyDelegate(CreateAlgoStrategy);

            // Set OnCreateIndicators delegate
            OnCreateIndicators = new OnCreateIndicatorsDelegate(CreateIndicators);

            // Set OnEachTickHandler for our on each tick strategy
            OnEachTickHandler = new OnTickDelegate(StrategyOnEachTickHandler);
        }

        /// <summary>
        /// Create MZpack Algo Strategy object for pattern and built-in position management (ATM).
        /// </summary>
        protected MZpack.NT8.Algo.Strategy CreateAlgoStrategy()
        {
            return new AlgoStrategy1(@"AlgoStrategy1", this) 
            { 
                OppositePatternAction = Common_OppositePatternAction,
                LogLevel = LogLevel, // Set log level of algo from strategy UI
                LogTarget = LogTarget, // Set log target of algo from strategy UI
                LogTime = LogTime
            };
        }

        /// <summary>
        /// Create indicators here and return them in the list as a result.
        /// </summary>
        /// <returns>Indicators list</returns>
        protected List<TickIndicator> CreateIndicators()
        {
            // Initialize new indicators list
            List<TickIndicator> indicators = new List<TickIndicator>();

            // Create BigTrade indicator
            BigTradeIndicator = new StrategyBigTradeIndicator(this, @"Big Trade")
            {
                // OrderType has depricated
                //OrderType = OrderFlowOrderType.Market,

                // Display filters
                FilterLogic = TradeFilterLogic.ALL,
                // Presentation
                MarkerType = TradeMarkerType.Box,
                ColorMode = ColorMode.Saturation,
                ShowTradePOC = true,
                VolumesPosition = VolumesPosition.OutsideRight,
                // Alerts
                UseBigTradeAlert = false
            };
            indicators.Add(BigTradeIndicator);

            EMAIndicator = new EMA(this, Close, EMA_Period);
            EMAIndicator.Strokes[0] = new Stroke(System.Windows.Media.Brushes.Yellow);
            EMAIndicator.Visible = true;
            indicators.Add(EMAIndicator);

            DOMImbalanceIndicator = new StrategyMarketDepthIndicator(this, @"Market Depth")
            {
                ShowRealtimeOrderBook = false,
                ShowImbalance = true,
                ImbalancePercentage = Filters_DOMImbalance_Ratio * 100d - 100d // Conver ratio to percentage, eg. 1.2 = 20%
            };
            indicators.Add(DOMImbalanceIndicator);

            return indicators;
        }

        protected override void OnStateChange()
        {
            // Base OnStateChange() call is required
            base.OnStateChange();

            lock (Sync) // Sync state handler
            {
                if (State == State.SetDefaults)
                {
                    // Depricated, is always Calculate.OnEachTick 
                    //Calculate = Calculate.OnEachTick; // OnEachTick strategy

                    // Default values for parameters
                    Common_EntryMethod = EntryMethod.Limit;
                    Common_CancelLimitAfterBars = 10;
                    Common_OppositePatternAction = OppositePatternAction.Reverse;
                    Common_TraceOrders = false;
                    // Position allowed directions, quantities and stops/targets
                    Position_Direction = SignalDirection.Any;
                    Position_Quantity1 = 2;
                    Position_Quantity2 = 1;
                    Position_StopLoss1 = 18;
                    Position_StopLoss2 = 20;
                    Position_ProfitTarget1 = 35;
                    Position_ProfitTarget2 = 70;
                    // Break even
                    Position_IsBE = false;
                    Position_BEAfterTicks = 20;
                    Position_BEShiftTicks = 5;
                    // Trailing
                    Position_IsTrail = true;
                    Position_TrailAfter = 15;
                    Position_TrailDistance = 10;
                    Position_TrailStep = 2;
                    // Signals
                    BigTrade_TradeVolume = 300;
                    BigTrade_IcebergVolume = 0;
                    BigTrade_IcebergAlgo = IcebergAlgo.Soft;
                    BigTrade_DomPressureVolume = 20;
                    EMA_Period = 14;
                    Signals_EntryShiftTicks = -2;
                    // Filters
                    Filters_DOMImbalance = true;
                    Filters_DOMImbalance_Ratio = 1.4;
                }
                else
                if (State == State.Configure)
                {
                    TraceOrders = Common_TraceOrders;
                    EntriesPerDirection = Position_Quantity2 > 0 ? 2 : 1;

                    // Assign indicator's attributes from UI properties of the strategy
                    BigTradeIndicator.TradeFilterMin = BigTrade_TradeVolume;
                    BigTradeIndicator.IcebergFilterMin = BigTrade_IcebergVolume;
                    BigTradeIndicator.IcebergAlgo = BigTrade_IcebergAlgo;
                    BigTradeIndicator.DomPressureFilterEnable = BigTrade_DomPressureVolume > 0;
                    BigTradeIndicator.DomPressureFilterMin = BigTrade_DomPressureVolume;
                    BigTradeIndicator.Visible = true;

                    // Create entries
                    Entry[] entries = new Entry[EntriesPerDirection];
                    entries[0] = new Entry(Strategy)
                    {
                        EntryMethod = Common_EntryMethod,
                        Quantity = Position_Quantity1,
                        SignalName = SIGNAL1,
                        StopLossTicks = Position_StopLoss1,
                        ProfitTargetTicks = Position_ProfitTarget1,
                        IsBreakEven = Position_IsBE,
                        BreakEvenAfterTicks = Position_BEAfterTicks,
                        BreakEvenShiftTicks = Position_BEShiftTicks
                    };

                    if (Position_Quantity2 > 0)
                        entries[1] = new Entry(Strategy)
                        {
                            EntryMethod = Common_EntryMethod,
                            Quantity = Position_Quantity2,
                            SignalName = SIGNAL2,
                            StopLossTicks = Position_StopLoss2,
                            ProfitTargetTicks = Position_ProfitTarget2,
                            IsBreakEven = Position_IsBE,
                            BreakEvenAfterTicks = Position_BEAfterTicks,
                            BreakEvenShiftTicks = Position_BEShiftTicks,
                            Trail = Position_IsTrail ? new Trail(Position_TrailAfter, Position_TrailDistance, Position_TrailStep) : null
                        };

                    // Pattern
                    Pattern pattern = new Pattern(Strategy,
                        Logic.And, // Logic of root node of signals tree
                        new Range() { Bars = 1 },  // Bar range for signals
                        Logic.And, // Logic of root node of filters tree
                        new Range() { Bars = 5 },  // Bar range for filters
                        true);  // Short-cuircit for AND evaluation

                    // Signals
                    pattern.Signals.Root.AddChild(new BigTradeSignal(Strategy) { Name = "Big Trade" });
                    pattern.Signals.Root.AddChild(new EMASignal(Strategy) { Name = "EMA" });

                    // Filters
                    if (Filters_DOMImbalance)
                        pattern.Filters.Root.AddChild(new DOMImbalanceFilter(Strategy, Filters_DOMImbalance_Ratio));

                    // Initialize strategy
                    Strategy.Initialize(pattern, entries, 3);
                }
            }
        }

        protected void StrategyOnEachTickHandler(MarketDataEventArgs e, int currentBarIdx)
        {
            lock (Sync) // Sync event handler
            {
                // Cancel pending orders if overpassed bars count exceeded
                if (pendingLimitBarIdx > 0 && currentBarIdx - pendingLimitBarIdx > Common_CancelLimitAfterBars && Strategy.Positions.HasPendingLimitOrders())
                {
                    Strategy.Pattern.Reset(true, false); // Reset pattern for position
                    Strategy.Positions.CancelPendingEntries("Cancelled by overpassed bars count", e.Time);
                    pendingLimitBarIdx = -1;
                }
            }
        }

        /// <summary>
        /// Override OnOrderUpdate() to add some strategy specific behavior.
        /// </summary>
        protected override void OnOrderUpdate(Order order, double limitPrice, double stopPrice, int quantity, int filled, double averageFillPrice, OrderState orderState, DateTime time, ErrorCode error, string nativeError)
        {
            base.OnOrderUpdate(order, limitPrice, stopPrice, quantity, filled, averageFillPrice, orderState, time, error, nativeError);

            if ((Strategy.Positions.FirstOrDefault().State & PositionState.LimitPending) > 0)  // Check state of the first and only position
                pendingLimitBarIdx = CurrentBar;
            else
                pendingLimitBarIdx = -1;
        }


        /// <summary>
        /// Signal by big trade.
        /// </summary>
        class BigTradeSignal : Signal
        {
            ITrade lastTrade;

            public BigTradeSignal(MZpack.NT8.Algo.Strategy strategy)
                : base(strategy, MarketDataSource.Level1, SignalCalculate.OnEachTick, false)
            {
            }

            public override void OnCalculate(MarketDataEventArgs e, int barIdx, SignalDirection allowed)
            {
                MZpackAlgoStrategy1 __strategy = ((MZpackAlgoStrategy1)Strategy.MZpackStrategy);

                // ReconstructedTrades contains all collected reconstructed trades without filtering
                if (__strategy.BigTradeIndicator.ReconstructedTrades.Count > 0)
                {
                    // Newest trade is the last trade in the trades list
                    ITrade trade = __strategy.BigTradeIndicator.ReconstructedTrades[__strategy.BigTradeIndicator.ReconstructedTrades.Count - 1];

                    // New trade has new unique trade Id
                    if (lastTrade == null || lastTrade.Id != trade.Id)
                    {
                        // Check trade filter
                        if (trade.Volume >= __strategy.BigTrade_TradeVolume && trade.IcebergVolume >= __strategy.BigTrade_IcebergVolume && trade.DomPressureVolume >= __strategy.BigTrade_DomPressureVolume)
                        {
                            Direction = trade.Side == TradeSide.Bid ? SignalDirection.Long : SignalDirection.Short; // Set signal direction
                            // Set chart range of the signal
                            ChartRange = new ChartRange() { MinBarIdx = trade.StartBarIdx, MaxBarIdx = trade.StopBarIdx };
                            // Set price of the signal
                            EntryPrice = __strategy.Instrument.MasterInstrument.RoundToTickSize(Direction == SignalDirection.Long
                                ? trade.StopPrice + Math.Min(__strategy.Signals_EntryShiftTicks * __strategy.TickSize, __strategy.GetCurrentBid())
                                : trade.StopPrice - Math.Min(__strategy.Signals_EntryShiftTicks * __strategy.TickSize, __strategy.GetCurrentAsk()));
                            // Add textual description for the log
                            Description = string.Format("{0} Big Trade signal: {1}", e.Time, trade); 
                        }

                        lastTrade = trade;
                    }
                }
            }
        }

        /// <summary>
        /// Signal based on EMA indicator. 
        /// </summary>
        class EMASignal : Signal
        {
            public EMASignal(MZpack.NT8.Algo.Strategy strategy)
                    : base(strategy, MarketDataSource.Level1, SignalCalculate.OnEachTick, false)
            {
            }

            public override void OnCalculate(MarketDataEventArgs e, int barIdx, SignalDirection allowed)
            {
                MZpackAlgoStrategy1 __strategy = ((MZpackAlgoStrategy1)Strategy.MZpackStrategy);

                double ema = __strategy.EMAIndicator.Values[0][barIdx];
                if ((e.Price > ema && Signal.IsShortAllowed(allowed)) ||
                    (e.Price < ema && Signal.IsLongAllowed(allowed)))
                {
                    // Price of the pattern is set by BigTradeSignal
                    Direction = allowed;
                    ChartRange = new ChartRange() { MinBarIdx = barIdx, MaxBarIdx = barIdx };  // Set chart range of the signal
                    Time = e.Time;
                    Description = string.Format("EMA: @{0}", ema);
                }
            }

            // Overrider is not required
            //public override void OnCalculate(MarketDepthEventArgs e, int barIdx, SignalDirection allowed)
            //{
            //}
        }

        /// <summary>
        /// Filter based on DOM Imbalance. 
        /// </summary>
        class DOMImbalanceFilter: Signal
        {
            double ratio;

            public DOMImbalanceFilter(MZpack.NT8.Algo.Strategy strategy, double ratio)
                    : base(strategy, MarketDataSource.Level2, SignalCalculate.OnEachTick, false)
            {
                this.ratio = ratio;
            }

            public override void OnCalculate(MarketDepthEventArgs e, int barIdx, SignalDirection allowed)
            {
                MZpackAlgoStrategy1 __strategy = ((MZpackAlgoStrategy1)Strategy.MZpackStrategy);

                if (barIdx > -1 && Signal.IsDetermined(allowed))  // mandatory signals conditions for this filter
                {
                    long totalOffer = __strategy.DOMImbalanceIndicator.RealtimeOrderBook.GetVolume(TradeSide.Ask);
                    long totalBid = __strategy.DOMImbalanceIndicator.RealtimeOrderBook.GetVolume(TradeSide.Bid);
                    double offerBidRatio = (double)totalOffer / totalBid;
                    double bidOfferRatio = (double)totalBid / totalOffer;

                    if ((Signal.IsShortAllowed(allowed) && offerBidRatio >= __strategy.Filters_DOMImbalance_Ratio) ||
                        (Signal.IsLongAllowed(allowed) && bidOfferRatio >= __strategy.Filters_DOMImbalance_Ratio))
                    {
                        // Price is not set because entry price is defined by the signals in this example.
                        Direction = allowed;
                        ChartRange = new ChartRange() { MinBarIdx = barIdx, MaxBarIdx = barIdx };  // Set chart range of the signal
                        Time = e.Time;
                        double ratio = Signal.IsShortAllowed(allowed) ? offerBidRatio : bidOfferRatio;
                        Description = string.Format("DOM Imbalance: {0}", ratio);
                    }
                }
            }
        }


        // --------------------------------- Common
        [Display(Name = "Order type", GroupName = "Common", Order = 1, Description = "")]
        public EntryMethod Common_EntryMethod
        { get; set; }

        [Display(Name = "Cancel Limit after, bars", GroupName = "Common", Order = 2, Description = "")]
        [Range(1, int.MaxValue)]
        public int Common_CancelLimitAfterBars
        { get; set; }

        [Display(Name = "On Opposite Signal", GroupName = "Common", Order = 20, Description = "")]
        public OppositePatternAction Common_OppositePatternAction
        { get; set; }
        
        [Display(Name = "Trace orders", GroupName = "Common", Order = 21, Description = "")]
        public bool Common_TraceOrders
        { get; set; }

        // ----------------------------- Position
        [Display(Name = "Direction", GroupName = "Position", Order = 0, Description = "")]
        public SignalDirection Position_Direction
        { get; set; }

        [Display(Name = "Quantity 1", GroupName = "Position", Order = 1, Description = "")]
        [Range(1, int.MaxValue)]
        public int Position_Quantity1
        { get; set; }

        [Display(Name = "Quantity 2", GroupName = "Position", Order = 2, Description = "")]
        [Range(0, int.MaxValue)]
        public int Position_Quantity2
        { get; set; }

        [Display(Name = "Stop loss 1, ticks", GroupName = "Position", Order = 3, Description = "")]
        [Range(1, int.MaxValue)]
        public int Position_StopLoss1
        { get; set; }

        [Display(Name = "Stop loss 2, ticks", GroupName = "Position", Order = 4, Description = "")]
        [Range(1, int.MaxValue)]
        public int Position_StopLoss2
        { get; set; }

        [Display(Name = "Profit target 1, ticks", GroupName = "Position", Order = 5, Description = "")]
        [Range(1, int.MaxValue)]
        public int Position_ProfitTarget1
        { get; set; }

        [Display(Name = "Profit target 2, ticks", GroupName = "Position", Order = 6, Description = "")]
        [Range(1, int.MaxValue)]
        public int Position_ProfitTarget2
        { get; set; }

        [Display(Name = "BE", GroupName = "Position", Order = 7, Description = "")]
        public bool Position_IsBE
        { get; set; } 

        [Display(Name = "BE after, ticks", GroupName = "Position", Order = 10, Description = "")]
        [Range(1, int.MaxValue)]
        public int Position_BEAfterTicks
        { get; set; }

        [Display(Name = "BE shift, ticks", GroupName = "Position", Order = 11, Description = "")]
        public int Position_BEShiftTicks
        { get; set; }

        [Display(Name = "Trail entry #2", GroupName = "Position", Order = 12, Description = "")]
        public bool Position_IsTrail
        { get; set; }

        [Display(Name = "Trail after, ticks", GroupName = "Position", Order = 20, Description = "")]
        [Range(1, int.MaxValue)]
        public int Position_TrailAfter
        { get; set; }

        [Display(Name = "Trail distance, ticks", GroupName = "Position", Order = 21, Description = "")]
        [Range(1, int.MaxValue)]
        public int Position_TrailDistance
        { get; set; }

        [Display(Name = "Trail step, ticks", GroupName = "Position", Order = 22, Description = "")]
        [Range(1, int.MaxValue)]
        public int Position_TrailStep
        { get; set; }

        //--------------------------------------- Signals
        [Display(Name = "BigTrade: trade volume", GroupName = "Signals", Order = 2, Description = "Trade volume filter")]
        [Range(1, long.MaxValue)]
        public long BigTrade_TradeVolume
        { get; set; } 

        [Display(Name = "BigTrade: iceberg volume", GroupName = "Signals", Order = 3, Description = "Iceberg volume filter")]
        [Range(0, long.MaxValue)]
        public long BigTrade_IcebergVolume
        { get; set; }

        [Display(Name = "BigTrade: iceberg algo", GroupName = "Signals", Order = 4, Description = "IcebergAlgo")]
        public IcebergAlgo BigTrade_IcebergAlgo
        { get; set; } 

        [Display(Name = "BigTrade: DOM pressure", GroupName = "Signals", Order = 5, Description = "Dom Pressure volume filter")]
        [Range(0, long.MaxValue)]
        public long BigTrade_DomPressureVolume
        { get; set; }

        [Display(Name = "EMA: period", GroupName = "Signals", Order = 10, Description = "")]
        [Range(1, int.MaxValue)]
        public int EMA_Period { get; set; }

        [Display(Name = "Entry shift, ticks", GroupName = "Signals", Order = 20, Description = "")]
        public int Signals_EntryShiftTicks
        { get; set; }

        //--------------------------------------- Filters
        [Display(Name = "DOM Imbalance", GroupName = "Filters", Order = 1, Description = "")]
        public bool Filters_DOMImbalance { get; set; }

        [Display(Name = "DOM Imbalance: ratio", GroupName = "Filters", Order = 2, Description = "")]
        public double Filters_DOMImbalance_Ratio { get; set; }

    }
}
#endif
