﻿// MZpack 3 Pro API demo
// Copyright © 2020 MZpack Co.
// www.mzpack.pro
// WARNING: to compile this sample in NinjaTrader 8 remove '#if APISAMPLE' and '#endif' directives
#if APISAMPLE
using System;
using System.Collections.Generic;
using NinjaTrader.Data;
using MZpack;
using MZpack.NT8;
using MZpack.NT8.Algo;
using MZpack.NT8.Algo.Indicators;
using System.ComponentModel.DataAnnotations;
using System.Linq;

namespace NinjaTrader.NinjaScript.Strategies.MZpackAPISamples
{
    /// <summary>
    /// Demonstrates how to use StrategyBigTradeIndicator class and StrategyVolumeProfileIndicator class in Custom mode, including backtesting in Strategy Analyzer.
    /// The logic of the strategy is 'On bar close'.
    /// ATTENTION: Indicator settings in UI are used only for displaying markers on the chart. Strategy uses its own internal settings for filters and logic.
    /// </summary>
    /// <summary>
    /// Strategy logic description.
    /// </summary>
    /// <summary>
    /// 1. Calculate bar range on bar close; 
    /// </summary>
    /// <summary>
    /// 2. If range of closed bar is greater or equal to 'Start bar range' start to build developing profile from this bar; 
    /// </summary>
    /// <summary>
    /// 3. If range of closed bar is less or equal to 'Min bar range' or greater that 'Max bar range', cancel building of profile;
    /// </summary>
    /// <summary>
    /// 3.1 Calculate delta of aggressive big trades inside profile;
    /// </summary>
    /// <summary>
    /// 3.2 Enter position: enter SHORT if Delta is negative and bigTradeDelta is positive; enter LONG if Delta is positive and bigTradeDelta is negative;
    /// </summary>
    /// <summary>
    /// See installation folder for the source code.
    /// </summary>
    public class MZpackCustomStrategy1 : MZpackStrategyBase
    {
        StrategyVolumeProfileIndicator volumeProfileIndicator;
        StrategyBigTradeIndicator bigTradeIndicator;
        IVolumeProfile profile; // Current volume profile

        string signalLong, signalShort;

        public MZpackCustomStrategy1() : base()
        {
            // Set OnCreateIndicators delegate
            OnCreateIndicators = new OnCreateIndicatorsDelegate(CreateIndicators);

            // Set OnBarCloseHandler for our on bar close strategy
            OnBarCloseHandler = new OnTickDelegate(StrategyOnBarCloseHandler);
        }

        /// <summary>
        /// Create indicators here and return them in the list as a result.
        /// </summary>
        /// <returns>Indicators list</returns>
        protected List<TickIndicator> CreateIndicators()
        {
            // Initialize new indicators list
            List<TickIndicator> indicators = new List<TickIndicator>();

            // Create CustomVolumeProfileIndicator instance with default settings
            volumeProfileIndicator = new StrategyVolumeProfileIndicator(this, @"Custom VP")
            {
                // Override defaults if required
                ProfileCreation = ProfileCreation.Custom,
                ProfileMode = ProfileMode.BuySell,
                StackedProfileCreation1 = ProfileCreation.Sessions,
                StackedProfileCreation2 = ProfileCreation.None,
                StackedProfileCreation3 = ProfileCreation.None
            };
            // Add indicator to the list
            indicators.Add(volumeProfileIndicator);

            // Create StrategyBigTradeIndicator instance with default settings
            bigTradeIndicator = new StrategyBigTradeIndicator(this, @"Aggressive Trades")
            {
                // Override defaults if required
                FilterLogic = TradeFilterLogic.ANY,
                MarkerType = TradeMarkerType.Bar,
                UseBigTradeAlert = false
            };
            indicators.Add(bigTradeIndicator);

            return indicators;
        }

        protected override void OnStateChange()
        {
            // Base OnStateChange() call is required
            base.OnStateChange();

            lock (Sync) // Sync state handler
            {
                if (State == State.SetDefaults)
                {
                    EntriesPerDirection = 1;

                    // Set defaults
                    Strategy_StackedProfileType = ProfileType.VP;
                    Strategy_TakeProfitTicks = 96;
                    Strategy_StopLossTicks = 32;
                    Strategy_StartProfileMinBarRange = 12;
                    Strategy_KeepProfileMinBarRange = 5;
                    Strategy_KeepProfileMaxBarRange = 30;
                    Strategy_TradeVolume = 100;
                    Strategy_AggressionFilter = 2;

                    // This is required for backtesting in Strategy Analyzer
                    EnableBacktesting = true;
                }
                else if (State == State.Configure)
                {
                    volumeProfileIndicator.Calculate = Calculate.OnBarClose;  // Calculate.OnBarClose is recommended for mzVolumeProfile for performance reasons if the logic of the strategy is OnBarClose

                    // Configure indicators from UI input

                    // When State = State.Configure we can set indicator attributes from UI properties
                    bigTradeIndicator.TradeFilterEnable = true;
                    bigTradeIndicator.TradeFilterMin = Strategy_TradeVolume;
                    bigTradeIndicator.TradeFilterMax = -1;
                    bigTradeIndicator.AggressionFilterEnable = true;
                    bigTradeIndicator.AggressionFilterMin = Strategy_AggressionFilter;
                    bigTradeIndicator.AggressionFilterMax = -1;
                    //
                    volumeProfileIndicator.TicksPerLevel = 4;
                    volumeProfileIndicator.StackedCreationN1 = 2;
                    volumeProfileIndicator.StackedShowProfileType[0] = Strategy_StackedProfileType;
                }
            }
        }

        // The handler triggered on bar close. currentBarIdx is the index of just closed bar.
        // e belongs to the next (just opened) bar.
        protected void StrategyOnBarCloseHandler(MarketDataEventArgs e, int currentBarIdx)
        {
            int barRange = 0;
            if (Bars.Count > 1)
                barRange = (int)((High[1] - Low[1]) / TickSize); // Calculate range of closed bar in ticks

            // Create new profile and exit position if any
            if (profile == null && barRange >= Strategy_StartProfileMinBarRange)
            {
                // Exit long if any
                if (!string.IsNullOrEmpty(signalLong))
                {
                    ExitLong("Cover", signalLong);
                    signalLong = null;
                }
                // Exit short if any
                if (!string.IsNullOrEmpty(signalShort))
                {
                    ExitShort("Cover.", signalShort);
                    signalShort = null;
                }

                profile = volumeProfileIndicator.AddProfile();  // Add new profile 
                profile.AssignRange(currentBarIdx, currentBarIdx + 1); // Start profile from the closed bar. Ending bar index is currentBarIdx + 1 at this point
                profile.IsDeveloping = true;  // Make it developing
            }
            else
            // Cancel building profile and enter position
            if (profile != null && (barRange <= Strategy_KeepProfileMinBarRange || profile.RangeTicks >= Strategy_KeepProfileMaxBarRange))
            {
                // Calculate big trades delta inside profile
                long bigTradeDelta = 0;
                List<ITrade> trades = bigTradeIndicator.Trades.Where(t => t.StartBarIdx >= profile.BeginBarIdx && t.StartBarIdx <= profile.EndBarIdx).ToList();  // Collect all filtered trades inside the profile

                foreach (ITrade trade in trades)
                {
                    if (trade.Side == TradeSide.Bid)
                        bigTradeDelta -= trade.Volume;  // Subtract Sell trade volume
                    else
                    if (trade.Side == TradeSide.Ask)
                        bigTradeDelta += trade.Volume;  // Add Buy trade volume
                }

                // Try enter position
                if (profile.Delta < 0 && bigTradeDelta > 0)
                {
                    signalShort = string.Format("VP delta:{0}, T delta:{1}", profile.Delta, bigTradeDelta);
                    SetProfitTarget(signalShort, CalculationMode.Ticks, Strategy_TakeProfitTicks);
                    SetStopLoss(signalShort, CalculationMode.Ticks, Strategy_StopLossTicks, false);
                    // Enter short if Delta is negative and bigTradeDelta is positive
                    EnterShort(signalShort);
                }
                else
                if (profile.Delta > 0 && bigTradeDelta < 0)
                {
                    signalLong = string.Format("VP delta:{0}, T delta:{1}", profile.Delta, bigTradeDelta);
                    SetProfitTarget(signalLong, CalculationMode.Ticks, Strategy_TakeProfitTicks);
                    SetStopLoss(signalLong, CalculationMode.Ticks, Strategy_StopLossTicks, false);
                    // Enter long if Delta is positive and bigTradeDelta is negative
                    EnterLong(signalLong);
                }

                // Cancel building of developing profile
                profile.IsDeveloping = false;
                profile = null;
            }
        }


        [Display(Name = "Take profit, ticks", GroupName = "00. Strategy", Order = 1, Description = "")]
        [Range(1, int.MaxValue)]
        [NinjaScriptProperty]
        public int Strategy_TakeProfitTicks
        { get; set; } 

        // Maximal stop loss
        [Display(Name = "Stop loss, ticks", GroupName = "00. Strategy", Order = 2, Description = "")]
        [Range(1, int.MaxValue)]
        [NinjaScriptProperty]
        public int Strategy_StopLossTicks
        { get; set; }

        /// <summary>
        /// Minimal bar range to create a profile
        /// </summary>
        [Display(Name = "Stacked profile type", GroupName = "01. Filtering", Order = 5, Description = "")]
        [NinjaScriptProperty]
        public ProfileType Strategy_StackedProfileType
        { get; set; }


        /// <summary>
        /// Minimal bar range to create a profile
        /// </summary>
        [Display(Name = "Start bar range", GroupName = "01. Filtering", Order = 10, Description = "Minimal bar range to create a profile")]
        [Range(1, long.MaxValue)]
        [NinjaScriptProperty]
        public int Strategy_StartProfileMinBarRange
        { get; set; }

        /// <summary>
        /// Minimal bar range to keep a profile
        /// </summary>
        [Display(Name = "Min bar range", GroupName = "01. Filtering", Order = 11, Description = "Minimal bar range to keep a profile")]
        [Range(1, long.MaxValue)]
        [NinjaScriptProperty]
        public int Strategy_KeepProfileMinBarRange
        { get; set; }

        /// <summary>
        /// Maximal bar range to keep a profile
        /// </summary>
        [Display(Name = "Max bar range", GroupName = "01. Filtering", Order = 12, Description = "Maximal bar range to keep a profile")]
        [Range(1, long.MaxValue)]
        [NinjaScriptProperty]
        public int Strategy_KeepProfileMaxBarRange
        { get; set; }

        /// <summary>
        /// BigTrade Indicator attribute
        /// </summary>
        [Display(Name = "Trade volume", GroupName = "01. Filtering", Order = 15, Description = "Trade volume filter")]
        [Range(0, long.MaxValue)]
        [NinjaScriptProperty]
        public long Strategy_TradeVolume
        { get; set; }

        /// <summary>
        /// BigTrade Indicator attribute
        /// </summary>
        [Display(Name = "Aggression/Sweep, ticks", GroupName = "01. Filtering", Order = 20, Description = "Aggression or sweep filter in ticks")]
        [Range(1, int.MaxValue)]
        [NinjaScriptProperty]
        public int Strategy_AggressionFilter
        { get; set; }
    }
}
#endif