﻿// MZpack 3 Pro API demo
// Copyright © 2020 MZpack Co.
// www.mzpack.pro
// WARNING: to compile this sample in NinjaTrader 8 remove '#if APISAMPLE' and '#endif' directives
#if APISAMPLE
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using MZpack.NT8;
using NinjaTrader.Gui;
using MZpack.NT8.Algo;
using MZpack.NT8.Algo.Indicators;
using NinjaTrader.Data;

namespace NinjaTrader.NinjaScript.Strategies.MZpackAPISamples
{
    /// <summary>
    /// Demonstrates how to use some of well known technical indicators.
    /// See API Documentation for the list of currently supported technical indicators.
    /// </summary>
    /// <summary>
    /// See installation folder for the source code.
    /// </summary>
    public class MZpackTechnicalSample0 : MZpackStrategyBase
    {
        EMA ema;
        CCI cci;
        ATR atr;
        DonchianChannel donchianChannel;


        public MZpackTechnicalSample0() : base()
        {
            // Set OnCreateIndicators delegate
            OnCreateIndicators = new OnCreateIndicatorsDelegate(CreateIndicators);

            // Set OnBarCloseHandler for our on bar close strategy
            OnBarCloseHandler = new OnTickDelegate(StrategyOnBarCloseHandler);
        }

        /// <summary>
        /// Create indicators here and return them in the list as a result.
        /// </summary>
        /// <returns>Indicators list</returns>
        protected List<TickIndicator> CreateIndicators()
        {
            // Initialize new indicators list
            List<TickIndicator> indicators = new List<TickIndicator>();

            // Create EMA indicator instance
            ema = new EMA(this, Close, EMA_Period);
            ema.Strokes[0] = new Stroke(System.Windows.Media.Brushes.Yellow);
            ema.PlotStyle = PlotStyle.Dot; // Override dafault PlotStyle.Line
            ema.Visible = true;
            indicators.Add(ema);

            // Create CCI indicator instance
            cci = new CCI(this, Close, CCI_Period);
            cci.Visible = false;  // Indicators can be only in Overlay mode. CCI values are always near zero, so indicator's plots are near zero too. Visible = false because normally we can't see plots near zero on chart panel.
            indicators.Add(cci);

            // ATR
            atr = new ATR(this, Close, 14);
            atr.Strokes[0] = new Stroke(System.Windows.Media.Brushes.Maroon);
            atr.Visible = true; 
            indicators.Add(atr);

            // Create DonchianChannel indicator instance
            donchianChannel = new DonchianChannel(this, Close, DonchianChannel_Period);
            donchianChannel.Strokes[0] = new Stroke(System.Windows.Media.Brushes.Blue);
            donchianChannel.Strokes[1] = new Stroke(System.Windows.Media.Brushes.Red);
            donchianChannel.Strokes[2] = new Stroke(System.Windows.Media.Brushes.Green);
            donchianChannel.Visible = true;
            indicators.Add(donchianChannel);

            return indicators;
        }

        protected override void OnStateChange()
        {
            // Base OnStateChange() call required
            base.OnStateChange();

            if (State == State.SetDefaults)
            {
                // Depricated, is always Calculate.OnEachTick 
                //Calculate = Calculate.OnEachTick;

                // Set defaults
                EMA_Period = 14;
                CCI_Period = 14;
                DonchianChannel_Period = 14;
            }
            else
            if (State == State.DataLoaded)
            {
                // Set working state of the strategy, ie. when OnEachTickHandler and OnBarCloseHandler trigger. 
                // True is required for backtesting in Strategy Analyzer.
                WorkingStateHistorical = true;
            }
        }

        protected void StrategyOnBarCloseHandler(MarketDataEventArgs e, int currentBarIdx)
        {
            // Access data series of an indicator using Values[] array
            // Values[0] is the first data series, etc.
            // Values in a data series are accessible by bar index (but not by ago value!)
            // Values[0][currentBarIdx] is the value for current bar, Values[0][currentBarIdx - 1] is the value for previous bar form data series #1
            // Values[1][currentBarIdx] is the value for current bar, Values[1][currentBarIdx - 1] is the value for previous bar form data series #2
            // Note, that currentBarIdx is the index of closed bar.

            Print(string.Format("{0}:  EMA = {1}  CCI = {2}  ATR = {3}",
                currentBarIdx,
                ema.Values[0].ContainsKey(currentBarIdx) ? ema.Values[0][currentBarIdx] : 0,
                cci.Values[0].ContainsKey(currentBarIdx) ? cci.Values[0][currentBarIdx] : 0,
                atr.Values[0].ContainsKey(currentBarIdx) ? atr.Values[0][currentBarIdx] : 0));
        }


        [Display(Name = "EMA period", GroupName = "NinjaScriptParameters", Order = 1, Description = "")]
        [Range(1, int.MaxValue)]
        [NinjaScriptProperty]
        public int EMA_Period
        { get; set; }

        [Display(Name = "CCI period", GroupName = "NinjaScriptParameters", Order = 2, Description = "")]
        [Range(1, int.MaxValue)]
        [NinjaScriptProperty]
        public int CCI_Period
        { get; set; }

        [Display(Name = "Donchian Channel period", GroupName = "NinjaScriptParameters", Order = 3, Description = "")]
        [Range(1, int.MaxValue)]
        [NinjaScriptProperty]
        public int DonchianChannel_Period
        { get; set; }

    }
}
#endif